@@ -1,8 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-
 """
-Describe SPEC #G control lines
+Describe SPEC diffractometer geometry #G control lines.
 
   #G0 ...       G[] array (geo mode, sector, etc)
   #G1 ...       U[] array (lattice constants, orientation reflections)
@@ -23,6 +20,7 @@ API
 """
 
 from collections import namedtuple, OrderedDict
+import datetime
 import logging
 import numpy
 import pathlib
@@ -37,11 +35,17 @@ KeyDescriptionValue = namedtuple(
     "KeyDescriptionValue", "key description value"
 )
 
-LatticeParameters = namedtuple(
-    "LatticeParameters", "a b c alpha beta gamma"
+LatticeParameters2D = namedtuple(
+    "LatticeParameters2D", "a b gamma"
 )
 
-Reflections = namedtuple("Reflections", "h k l wavelength angles")
+LatticeParameters3D = namedtuple(
+    "LatticeParameters3D", "a b c alpha beta gamma"
+)
+
+Reflections2D = namedtuple("Reflections2D", "h k wavelength angles")
+
+Reflections3D = namedtuple("Reflections3D", "h k l wavelength angles")
 
 
 # lattice constants, orientation reflections
@@ -105,6 +109,8 @@ class Diffractometer:
     .. autosummary::
 
         ~parse
+        ~print_all
+        ~print_brief
     """
 
     def __init__(self, geo_name):
@@ -134,6 +140,93 @@ class Diffractometer:
             self.variant or "",
         )
 
+    def _get_info_dict(self, level="str"):
+        """
+        Return dictionary for use in various reports.
+
+        PARAMETER
+
+        level str :
+            Level of detail for desired report.
+            One of these values: ``pa``, ``str``, ``wh``.
+            Default: ``str``
+        """
+        info = {}
+        gpars = self.geometry_parameters
+
+        info["geometry"] = f"'{self.geometry_name}'"
+        info["wavelength"] = f"{self.wavelength}"
+        info["mode"] = f"'{self.mode}'"
+        info["sector"] = f"{self.sector}"
+        for key in self._Q_names:
+            info[key] = f"{gpars[key.upper()].value}"
+
+        if level in ("pa", "wh"):
+            info["lattice"] = f"{self.lattice}"
+            for key in "alpha beta azimuth omega".split():  # TODO: generalize
+                if key.upper() in gpars:
+                    info[key.lower()] = f"{gpars[key.upper()].value}"
+
+        if level in ("pa", ):
+            full_name = self.geometry_name_full
+            info["full_geometry_name"] = f"'{full_name}'"
+            if hasattr(self, "lattice") and self.lattice is not None:
+                info["lattice"] = f"{self.lattice}"
+            if hasattr(self, "UB"):
+                info["UB"] = self.UB
+            if hasattr(self, "reflections"):
+                for i, refl in enumerate(self.reflections, start=1):
+                    info[f"reflection {i}"] = f"{refl}"
+                info["UB"] = self.UB
+
+        return info
+
+    def __str__(self):
+        """
+        Brief representation of this diffractometer.
+        """
+        s = [f"{k}={v}" for k, v in self._get_info_dict().items()]
+        return f"Diffractometer({', '.join(s)})"
+
+    @property
+    def _Q_names(self):
+        """
+        List of names used to define Q (typically: h k l).  All lower case.
+        """
+        geometry = get_geometry_catalog().get(self.geometry_name_full)
+        return [
+            q_term[0].lower()
+            for q_term in geometry["Q"]
+            if "Miller index" in q_term[1]
+        ]
+
+    def _positioners_dict(self, scan):
+        """
+        Dictionary with positioners (keys) and positions (values) in this scan.
+
+        PARAMETERS
+
+        scan obj :
+            SPEC data file scan (a ``spec2nexus.spec.SpecDataFileScan`` object).
+        """
+        geometry = get_geometry_catalog().get(self.geometry_name_full)
+        number_motors_in_geometry = geometry["numgeo"]
+        return {
+            k: scan.positioner[k]
+            for k in list(scan.positioner.keys())[:number_motors_in_geometry]
+        }
+
+    def _get_Q_dict(self, info):
+        """
+        Dictionary with all the Q-related terms & values (hkl, for example).
+
+        PARAMETERS
+
+        info dict :
+            from `_get_info_dict()`
+        """
+        return {key: info[key] for key in self._Q_names}
+
     def parse(self, scan):
         dgc = DiffractometerGeometryCatalog()
         gonio = dgc.get(self.geometry_name_full)
@@ -182,14 +275,12 @@ class Diffractometer:
                         for v, kd in zip(g1, gonio_U)
                     }
                 )
-                self.lattice = LatticeParameters(
-                    U["g_aa"].value,
-                    U["g_bb"].value,
-                    U["g_cc"].value,
-                    U["g_al"].value,
-                    U["g_be"].value,
-                    U["g_ga"].value,
-                )
+                if ("g_cc" in U) and ("g_al" in U) and ("g_be" in U):
+                    _parms = [U[f"g_{key}"].value for key in "aa bb cc al be ga".split()]
+                    self.lattice = LatticeParameters3D(*_parms)
+                else:  # twoc
+                    _parms = [U[f"g_{key}"].value for key in "aa bb ga".split()]
+                    self.lattice = LatticeParameters2D(*_parms)
                 for ref_num in (0, 1):
                     template = "g_u%d%%d" % ref_num
                     variant = dgc.get_variant(gonio, self.variant)
@@ -198,13 +289,17 @@ class Diffractometer:
                         k = template % i
                         if k in U:
                             angles[mne] = U[k].value
-                    ref = Reflections(
-                        U["g_h%d" % ref_num].value,
-                        U["g_k%d" % ref_num].value,
-                        U["g_l%d" % ref_num].value,
-                        U["g_lambda%d" % ref_num].value,
-                        angles,
-                    )
+                    if ("g_lambda" not in U):
+                        _parms = [U[f"g_{k}{ref_num}"].value for k in "h k l lambda".split()]
+                        _parms.append(angles)
+                        ref = Reflections3D(*_parms)
+                    else:  # twoc
+                        if ref_num > 0:
+                            break  # at most, 1 reflection is defined
+                        _parms = [U[f"g_{k}{ref_num}"].value for k in "h k".split()]
+                        _parms.append(U["g_lambda"].value)
+                        _parms.append(angles)
+                        ref = Reflections2D(*_parms)
                     self.reflections.append(ref)
 
                 self.geometry_parameters.update(U)
@@ -216,6 +311,7 @@ class Diffractometer:
                 self.geometry_parameters[
                     "ub_matrix"
                 ] = KeyDescriptionValue("ub_matrix", "UB[] matrix", UB)
+                self.UB = UB
 
         Q = OrderedDict()
         if "G4" in scan.G:
@@ -231,6 +327,67 @@ class Diffractometer:
                     self.wavelength = Q["LAMBDA"].value
         self.geometry_parameters.update(Q)
 
+    def print_all(self, scan):
+        """
+        Print All (pa) about this diffractometer scan.
+
+        PARAMETERS
+
+        scan obj :
+            SPEC data file scan (a ``spec2nexus.spec.SpecDataFileScan`` object).
+        """
+        scan.interpret()
+        s = {
+            "SPEC file": scan.specFile,
+            "scan #": scan.scanNum,
+            "SPEC scanCmd": scan.scanCmd,
+        }
+        if hasattr(scan, "epoch"):
+            s["date"] = datetime.datetime.fromtimestamp(scan.epoch)
+        s.update(self._get_info_dict("pa"))
+        s.update(self._positioners_dict(scan))
+
+        width = max([len(k) for k in s.keys()])
+        return '\n'.join([f"{k:^{width}} = {v}" for k, v in s.items()])
+
+    def print_brief(self, scan):
+        """
+        Print brief information (wh: where) about this diffractometer scan.
+
+        PARAMETERS
+
+        scan obj :
+            SPEC data file scan (a ``spec2nexus.spec.SpecDataFileScan`` object).
+        """
+        info = self._get_info_dict("wh")
+        qdict = self._get_Q_dict(info)
+
+        s = []
+
+        s.append(info["geometry"].strip("'"))
+
+        if len(qdict):
+            label = f"{' '.join([k for k in qdict.keys()])}"
+            r = f"{'  '.join(qdict.values())}"
+            s.append(f"{label} = {r}")
+
+        def _compose_row_(labels):
+            return "  ".join(
+                [
+                    f"{k}={info[k]}"
+                    for k in labels
+                    if k in info
+                ]
+            )
+
+        s.append(_compose_row_("alpha beta azimuth".split()))
+        s.append(_compose_row_("omega wavelength".split()))
+
+        for k, v in self._positioners_dict(scan).items():
+            s.append(f"{k} = {v}")
+
+        return '\n'.join(s)
+
 
 # singleton reference to DiffractometerGeometryCatalog
 _geometry_catalog = None
